//
//  ViewController.swift
//  Sense2
//
//  Created by Daniel Hauagge on 2/3/16.
//  Copyright © 2016 Daniel Hauagge. All rights reserved.
//

import UIKit
import CoreMotion

class ViewController: UIViewController {
    
    var motionManager = CMMotionManager()
            
    @IBOutlet weak var imageView: ImageView!
    
    var currPoint = CGPointMake(0,0)
    var clearDrawing = false
    var isDrawing = false
    
    var red: CGFloat = 1.0
    var green: CGFloat = 0
    var blue: CGFloat = 0
    
    var useForceTouch = true
    var brushRadius = CGFloat(5.0)
    
    var updateInterval: Double = 1.0 / 60.0 // seconds
    
    func processData() {}
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        imageView.userInteractionEnabled = true
        imageView.delegate = self
    
        motionManager.accelerometerUpdateInterval = updateInterval
        motionManager.startAccelerometerUpdatesToQueue(NSOperationQueue.mainQueue()) {
            (motion: CMAccelerometerData?, error: NSError?) in

            let minX = -self.imageView.frame.size.width/2.0 + CGFloat(self.brushRadius/2.0)
            let maxX = +self.imageView.frame.size.width/2.0 - CGFloat(self.brushRadius/2.0)
            let minY = -self.imageView.frame.size.height/2.0 + CGFloat(self.brushRadius/2.0)
            let maxY =  self.imageView.frame.size.height/2.0 - CGFloat(self.brushRadius/2.0)
            
            // Update the location of the cursor
            let scale = CGFloat(5)
            var nextPoint = CGPointMake(
                self.currPoint.x + scale * CGFloat(motion!.acceleration.x),
                self.currPoint.y + scale * -CGFloat(motion!.acceleration.y))
    
            // Make sure point is in bounds
            nextPoint.x = min(max(nextPoint.x, minX), maxX)
            nextPoint.y = min(max(nextPoint.y, minY), maxY)
            
            // Draw the line
            self.drawLine(fromPoint: self.currPoint, toPoint: nextPoint)
            self.currPoint = nextPoint
            self.imageView.cursorPosition = self.currPoint
        }
    }
    
    func drawLine(fromPoint a: CGPoint, toPoint b:CGPoint) {
        if !self.isDrawing {
            return
        }
        
        let imSize = self.imageView.frame.size
        
        UIGraphicsBeginImageContext(imSize)
        
        let context = UIGraphicsGetCurrentContext()
        
        self.imageView.image?.drawInRect(
            CGRectMake(0, 0,
                imSize.width,
                imSize.height)
        )
        
        let origin = CGPointMake(imSize.width/2.0, imSize.height/2.0)
        
        // Draw line
        CGContextMoveToPoint(context,    a.x + origin.x, a.y + origin.y)
        CGContextAddLineToPoint(context, b.x + origin.x, b.y + origin.y)
        
        // Set line params
        CGContextSetLineWidth(context, brushRadius)
        CGContextSetRGBStrokeColor(context, self.red, self.green, self.blue, 1.0)
        CGContextSetLineCap(context, .Round)
        
        // Render the drawing
        CGContextStrokePath(context)
        
        // Get the newly rendered image into the image view
        self.imageView.image = UIGraphicsGetImageFromCurrentImageContext()

        // End the graphics context
        UIGraphicsEndImageContext()
        
        self.imageView.setNeedsDisplay()
    }

    // This function is executed right after the user clicks on the "Settings"
    // button
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        let settings = segue.destinationViewController as! SettingsViewController
        
        // This is necessary to get the data back from the settings view
        settings.delegate = self
        
        // Send the current color to the settings view
        settings.red = self.red
        settings.green = self.green
        settings.blue = self.blue
        
        settings.useForceTouch = useForceTouch
        settings.brushRadius = brushRadius
    }

    @IBAction func clearButtonPressed(sender: AnyObject) {
        currPoint = CGPointMake(0, 0)
        self.imageView.resetImage()
        self.imageView.setNeedsDisplay()
    }
}

extension ViewController: SettingsViewControllerDelegate {
    // This method is called by the settings view
    
    func settingsViewControllerFinished(red : CGFloat, green : CGFloat, blue : CGFloat, brushRadius: CGFloat, useForceTouch: Bool)
    {
        self.red = red
        self.green = green
        self.blue = blue
        
        self.useForceTouch = useForceTouch
        if !useForceTouch {
            self.brushRadius = brushRadius
        }
    }
}

extension ViewController: ImageViewDelegate {
    func touchBegan(withForce force: CGFloat) {
        
        self.isDrawing = true
        if self.useForceTouch {
            self.brushRadius = CGFloat(5.0) * max(CGFloat(0.0), force)
        }
    }
    
    func touchMoved(withForce force: CGFloat) {
        if !self.useForceTouch { return }
        
        if self.useForceTouch {
            self.brushRadius = CGFloat(5.0) * max(CGFloat(0.0), force)
        }
    }
    
    func touchEnded() {
        self.isDrawing = false
    }
}
